@@ -1,4 +1,4 @@
-/* $Id: files.c 4307 2008-08-17 16:31:11Z astyanax $ */
+/* $Id: files.c 4231 2008-03-16 14:24:28Z astyanax $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
@@ -700,11 +700,14 @@ void do_insertfile(
     filestruct *edittop_save = openfile->edittop;
     size_t current_x_save = openfile->current_x;
     ssize_t current_y_save = openfile->current_y;
-    bool edittop_inside = FALSE;
+    bool edittop_inside = FALSE, meta_key = FALSE, func_key = FALSE;
+    const sc *s;
 #ifndef NANO_TINY
     bool right_side_up = FALSE, single_line = FALSE;
 #endif
 
+    currmenu = MINSERTFILE;
+
     while (TRUE) {
 #ifndef NANO_TINY
 	if (execute) {
@@ -731,12 +734,11 @@ void do_insertfile(
 		TRUE,
 #endif
 #ifndef NANO_TINY
-		execute ? extcmd_list :
+		execute ? MEXTCMD :
 #endif
-		insertfile_list, ans,
-#ifndef NANO_TINY
+		MINSERTFILE, ans,
+		&meta_key, &func_key,
 		NULL,
-#endif
 		edit_refresh, msg,
 #ifndef DISABLE_OPERATINGDIR
 		operating_dir != NULL && strcmp(operating_dir,
@@ -760,16 +762,19 @@ void do_insertfile(
 
 	    ans = mallocstrcpy(ans, answer);
 
+	    s = get_shortcut(currmenu, &i, &meta_key, &func_key);
+
 #ifndef NANO_TINY
 #ifdef ENABLE_MULTIBUFFER
-	    if (i == TOGGLE_MULTIBUFFER_KEY) {
+
+	    if (s && s->scfunc == (void *) new_buffer_msg) {
 		/* Don't allow toggling if we're in view mode. */
 		if (!ISSET(VIEW_MODE))
 		    TOGGLE(MULTIBUFFER);
 		continue;
 	    } else
 #endif
-	    if (i == NANO_TOOTHERINSERT_KEY) {
+	    if (s && s->scfunc == (void *) ext_cmd_msg) {
 		execute = !execute;
 		continue;
 	    }
@@ -779,7 +784,7 @@ void do_insertfile(
 #endif /* !NANO_TINY */
 
 #ifndef DISABLE_BROWSER
-	    if (i == NANO_TOFILES_KEY) {
+	    if (s && s->scfunc == (void *) to_files_msg) {
 		char *tmp = do_browse_from(answer);
 
 		if (tmp == NULL)
@@ -949,6 +954,7 @@ void do_insertfile(
 	    break;
 	}
     }
+    shortcut_init(FALSE);
 
     free(ans);
 }
@@ -1564,7 +1570,7 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 	}
 
 	if (f_open == NULL) {
-	    fd_source = open(realname, O_RDONLY | O_CREAT, S_IRUSR | S_IWUSR);
+	    fd_source = open(realname, O_RDONLY | O_CREAT);
 
 	    if (fd_source != -1) {
 		f_source = fdopen(fd_source, "rb");
@@ -1683,7 +1689,7 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 	int fd_source;
 	FILE *f_source = NULL;
 
-	fd_source = open(tempname, O_RDONLY | O_CREAT, S_IRUSR | S_IWUSR);
+	fd_source = open(tempname, O_RDONLY | O_CREAT);
 
 	if (fd_source != -1) {
 	    f_source = fdopen(fd_source, "rb");
@@ -1704,11 +1710,8 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 		strerror(errno));
 	    goto cleanup_and_exit;
 	}
-    } else if (fclose(f) != 0) {
-	    statusbar(_("Error writing %s: %s"), realname,
-		strerror(errno));
-	    goto cleanup_and_exit;
-    }
+    } else
+	fclose(f);
 
     if (!tmp && append == OVERWRITE) {
 	if (!nonamechange) {
@@ -1815,9 +1818,10 @@ bool do_writeout(bool exiting)
 #ifdef NANO_EXTRA
     static bool did_credits = FALSE;
 #endif
-    bool retval = FALSE;
+    bool retval = FALSE, meta_key = FALSE, func_key = FALSE;
+    const sc *s;
 
-    currshortcut = writefile_list;
+    currmenu = MWRITEFILE;
 
     if (exiting && openfile->filename[0] != '\0' && ISSET(TEMP_FILE)) {
 	retval = write_file(openfile->filename, NULL, FALSE, OVERWRITE,
@@ -1868,7 +1872,8 @@ bool do_writeout(bool exiting)
 #ifndef DISABLE_TABCOMP
 		TRUE,
 #endif
-		writefile_list, ans,
+		MWRITEFILE, ans,
+		&meta_key, &func_key,
 #ifndef NANO_TINY
 		NULL,
 #endif
@@ -1888,9 +1893,10 @@ bool do_writeout(bool exiting)
 	    break;
 	} else {
 	    ans = mallocstrcpy(ans, answer);
+            s = get_shortcut(currmenu, &i, &meta_key, &func_key);
 
 #ifndef DISABLE_BROWSER
-	    if (i == NANO_TOFILES_KEY) {
+	    if (s && s->scfunc == (void *) to_files_msg) {
 		char *tmp = do_browse_from(answer);
 
 		if (tmp == NULL)
@@ -1902,23 +1908,23 @@ bool do_writeout(bool exiting)
 	    } else
 #endif /* !DISABLE_BROWSER */
 #ifndef NANO_TINY
-	    if (i == TOGGLE_DOS_KEY) {
+	    if (s && s->scfunc == (void *) dos_format_msg) {
 		openfile->fmt = (openfile->fmt == DOS_FILE) ? NIX_FILE :
 			DOS_FILE;
 		continue;
-	    } else if (i == TOGGLE_MAC_KEY) {
+	    } else if (s && s->scfunc == (void *) mac_format_msg) {
 		openfile->fmt = (openfile->fmt == MAC_FILE) ? NIX_FILE :
 			MAC_FILE;
 		continue;
-	    } else if (i == TOGGLE_BACKUP_KEY) {
+	    } else if (s && s->scfunc == (void *) backup_file_msg) {
 		TOGGLE(BACKUP_FILE);
 		continue;
 	    } else
 #endif /* !NANO_TINY */
-	    if (i == NANO_PREPEND_KEY) {
+	    if (s && s->scfunc == (void *) prepend_msg) {
 		append = (append == PREPEND) ? OVERWRITE : PREPEND;
 		continue;
-	    } else if (i == NANO_APPEND_KEY) {
+	    } else if (s && s->scfunc == (void *) append_msg) {
 		append = (append == APPEND) ? OVERWRITE : APPEND;
 		continue;
 	    }
@@ -2387,7 +2393,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	} else if (!*lastwastab || num_matches < 2)
 	    *lastwastab = TRUE;
 	else {
-	    int longest_name = 0, ncols, editline = 0;
+	    int longest_name = 0, columns, editline = 0;
 
 	    /* Now we show a list of the available choices. */
 	    assert(num_matches > 1);
@@ -2412,7 +2418,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	    /* Each column will be (longest_name + 2) columns wide, i.e.
 	     * two spaces between columns, except that there will be
 	     * only one space after the last column. */
-	    ncols = (COLS + 1) / (longest_name + 2);
+	    columns = (COLS + 1) / (longest_name + 2);
 
 	    /* Blank the edit window, and print the matches out
 	     * there. */
@@ -2426,11 +2432,11 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 		char *disp;
 
 		wmove(edit, editline, (longest_name + 2) *
-			(match % ncols));
+			(match % columns));
 
-		if (match % ncols == 0 &&
+		if (match % columns == 0 &&
 			editline == editwinrows - 1 &&
-			num_matches - match > ncols) {
+			num_matches - match > columns) {
 		    waddstr(edit, _("(more)"));
 		    break;
 		}
@@ -2440,7 +2446,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 		waddstr(edit, disp);
 		free(disp);
 
-		if ((match + 1) % ncols == 0)
+		if ((match + 1) % columns == 0)
 		    editline++;
 	    }
 
