@@ -1,4 +1,4 @@
-/* $Id: proto.h 4261 2008-05-31 23:12:34Z astyanax $ */
+/* $Id: proto.h 4226 2008-03-14 04:08:51Z astyanax $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -91,34 +91,15 @@ extern char *full_operating_dir;
 extern char *alt_speller;
 #endif
 
-extern shortcut *main_list;
-extern shortcut *whereis_list;
-extern shortcut *replace_list;
-extern shortcut *replace_list_2;
-extern shortcut *gotoline_list;
-extern shortcut *writefile_list;
-extern shortcut *insertfile_list;
-#ifndef NANO_TINY
-extern shortcut *extcmd_list;
-#endif
-#ifndef DISABLE_HELP
-extern shortcut *help_list;
-#endif
-#ifndef DISABLE_SPELLER
-extern shortcut *spell_list;
-#endif
-#ifndef DISABLE_BROWSER
-extern shortcut *browser_list;
-extern shortcut *whereis_file_list;
-extern shortcut *gotodir_list;
-#endif
-
+extern sc *sclist;
+extern subnfunc *allfuncs;
 #ifdef ENABLE_COLOR
 extern syntaxtype *syntaxes;
 extern char *syntaxstr;
 #endif
 
 extern const shortcut *currshortcut;
+extern int currmenu;
 #ifndef NANO_TINY
 extern toggle *toggles;
 #endif
@@ -341,7 +322,7 @@ void save_history(void);
 #endif
 
 /* All functions in global.c. */
-size_t length_of_list(const shortcut *s);
+size_t length_of_list(int menu);
 #ifndef NANO_TINY
 void toggle_init_one(int val
 #ifndef DISABLE_HELP
@@ -472,7 +453,7 @@ RETSIGTYPE handle_sigwinch(int signal);
 void allow_pending_sigwinch(bool allow);
 #endif
 #ifndef NANO_TINY
-void do_toggle(const toggle *which);
+void do_toggle(int flag);
 #endif
 void disable_extended_io(void);
 #ifdef USE_SLANG
@@ -484,7 +465,7 @@ void enable_signals(void);
 void disable_flow_control(void);
 void enable_flow_control(void);
 void terminal_init(void);
-int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
+int do_input(bool *meta_key, bool *func_key, bool *have_shortcut, bool
 	*ran_func, bool *finished, bool allow_funcs);
 #ifndef DISABLE_MOUSE
 int do_mouse(void);
@@ -492,7 +473,7 @@ int do_mouse(void);
 void do_output(char *output, size_t output_len, bool allow_cntrls);
 
 /* All functions in prompt.c. */
-int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
+int do_statusbar_input(bool *meta_key, bool *func_key, bool *have_shortcut,
 	bool *ran_func, bool *finished, bool allow_funcs, void
 	(*refresh_func)(void));
 #ifndef DISABLE_MOUSE
@@ -523,15 +504,16 @@ void reset_statusbar_cursor(void);
 void update_statusbar_line(const char *curranswer, size_t index);
 bool need_statusbar_horizontal_update(size_t pww_save);
 void total_statusbar_refresh(void (*refresh_func)(void));
-int get_prompt_string(bool allow_tabs,
+const sc *get_prompt_string(int *value, bool allow_tabs,
 #ifndef DISABLE_TABCOMP
 	bool allow_files,
 #endif
 	const char *curranswer,
+	bool *meta_key, bool *func_key,
 #ifndef NANO_TINY
 	filestruct **history_list,
 #endif
-	void (*refresh_func)(void), const shortcut *s
+	void (*refresh_func)(void), int menu
 #ifndef DISABLE_TABCOMP
 	, bool *list
 #endif
@@ -540,7 +522,8 @@ int do_prompt(bool allow_tabs,
 #ifndef DISABLE_TABCOMP
 	bool allow_files,
 #endif
-	const shortcut *s, const char *curranswer,
+	int menu, const char *curranswer,
+	bool *meta_key, bool *func_key,
 #ifndef NANO_TINY
 	filestruct **history_list,
 #endif
@@ -649,7 +632,7 @@ bool do_wrap(filestruct *line);
 #if !defined(DISABLE_HELP) || !defined(DISABLE_WRAPJUSTIFY)
 ssize_t break_line(const char *line, ssize_t goal
 #ifndef DISABLE_HELP
-	, bool newln
+	, bool newline
 #endif
 	);
 #endif
@@ -753,8 +736,9 @@ int *parse_verbatim_kbinput(WINDOW *win, size_t *kbinput_len);
 #ifndef DISABLE_MOUSE
 int get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts);
 #endif
-const shortcut *get_shortcut(const shortcut *s_list, int *kbinput, bool
+const sc *get_shortcut(int menu, int *kbinput, bool
 	*meta_key, bool *func_key);
+const sc *first_sc_for(int menu, void *func);
 #ifndef NANO_TINY
 const toggle *get_toggle(int kbinput, bool meta_key);
 #endif
@@ -770,7 +754,7 @@ char *display_string(const char *buf, size_t start_col, size_t len, bool
 void titlebar(const char *path);
 void set_modified(void);
 void statusbar(const char *msg, ...);
-void bottombars(const shortcut *s);
+void bottombars(int menu);
 void onekey(const char *keystroke, const char *desc, size_t len);
 void reset_cursor(void);
 void edit_draw(const filestruct *fileptr, const char *converted, int
@@ -788,6 +772,49 @@ void display_main_list(void);
 void do_cursorpos(bool constant);
 void do_cursorpos_void(void);
 void do_replace_highlight(bool highlight, const char *word);
+char *flagtostr(int flag);
+const subnfunc *sctofunc(sc *s);
+void print_sclist(void);
+sc *strtosc(int menu, char *input);
+function_type strtokeytype(char *str);
+int strtomenu(char *input);
+void assign_keyinfo(sc *s);
+void xon_complaint(void);
+void xoff_complaint(void);
+int sc_seq_or (void *func, int defaultval);
+void do_suspend_void(void);
+
+const char *cancel_msg;
+#ifndef NANO_TINY
+const char *case_sens_msg;
+const char *backwards_msg;
+#endif
+const char *replace_msg;
+const char *no_replace_msg;
+const char *go_to_line_msg;
+const char *whereis_next_msg;
+const char *first_file_msg;
+const char *last_file_msg;
+const char *goto_dir_msg;
+const char *ext_cmd_msg;
+const char *to_files_msg;
+const char *dos_format_msg;
+const char *mac_format_msg;
+const char *append_msg;
+const char *prepend_msg;
+const char *backup_file_msg;
+const char *dos_format_msg;
+const char *mac_format_msg;
+const char *append_msg;
+const char *prepend_msg;
+const char *backup_file_msg;
+const char *gototext_msg;
+const char *new_buffer_msg;
+
+#ifdef HAVE_REGEX_H
+const char *regexp_msg;
+#endif
+
 #ifdef NANO_EXTRA
 void do_credits(void);
 #endif
